@@ -52,7 +52,7 @@ is.marssMODEL <- function(MODELobj, method = "kem") {
   ###########################
   # Check that the attributes are complete and consistent
   ###########################
-  el <- c("form", "model.dims", "par.names", "X.names", "Y.names", "equation", "obj.elements")
+  el <- c("form", "model.dims", "par.names", "X.names", "Y.names", "equation", "obj.elements", "model.tsp")
   attr.names <- names(attributes(MODELobj))
   if (!all(el %in% attr.names)) {
     msg <- c(msg, "Element", el[!(el %in% attr.names)], "is missing from the attributes of the model object.\n")
@@ -247,6 +247,14 @@ is.marssMODEL <- function(MODELobj, method = "kem") {
     msg <- c(msg, paste("The rownames of the data and the attribute Y.names don't match.\n", sep = ""))
   }
 
+  ###########################
+  # tsp is present and right length
+  ###########################
+  model.tsp <- attr(MODELobj, "model.tsp")
+  if ((model.tsp[2] - model.tsp[1]) * model.tsp[3] + 1 != TT) {
+    msg <- c(msg, paste("The model tsp attribute and the data length do not match.\n", sep = ""))
+  }
+
   ###########################
   # Check tinitx; must be 0 or 1
   ###########################
